/*
** auth_file.c
**
** Copyright (c) 1994-1996 Peter Eriksson <pen@signum.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <string.h>
#include <crypt.h>

#include "phttpd.h"


static hashtable_t *auth_files_table = NULL;


static struct options authfile_cfg_table[] =
{
    { "auth-files",	T_HTABLE, &auth_files_table,  NULL },
    { NULL,             -1,       NULL,		NULL }
};


int pm_init(const char **argv)
{
    char *cfg_path, *cp;
    const char *name = argv[0];
    
    if (debug > 1)
	fprintf(stderr, "*** auth_file/pm_init(\"%s\") called ***\n", name);

    cfg_path = s_malloc(strlen(name)+6);
    strcpy(cfg_path, name);
    
    cp = strrchr(cfg_path, '.');
    if (cp && strcmp(cp, ".so") == 0)
	*cp = '\0';
    
    strcat(cfg_path, ".conf");
    if (config_parse_file(cfg_path, authfile_cfg_table, 0) < 0)
	return -1;
    
    if (config_parse_argv(argv+1, authfile_cfg_table) < 0)
	return -1;
    
    return 0;
}


void pm_exit(void)
{
    if (debug > 1)
	fprintf(stderr, "*** auth_file/pm_exit() called ***\n");
    
    ht_destroy(auth_files_table);
    s_free(auth_files_table);
}


static int check_file(const char *path,
		      const char *user,
		      const char *password,
		      struct connectioninfo *cip)
{
    int fd;
    char buf[1024], *up, *pp, *lp;
    

    if (debug > 2)
	fprintf(stderr, "auth_file: check_file: %s: user=%s, password=%s.\n",
		path, user, password);
    
    fd = fd_open(path, O_RDONLY);
    if (fd < 0)
	return 0;

    while (fd_gets(buf, sizeof(buf), fd))
    {
	up = strtok_r(buf, " \t\n\r", &lp);
	if (up == NULL || up[0] == '#')
	    continue;

	if (strcmp(up, user) == 0)
	{
	    if (debug > 2)
		fprintf(stderr, "auth_file: check_file: Found user\n");
	    
	    pp = strtok_r(NULL, "\n\r", &lp);
	    if (pp == NULL)
	    {
		if (password == NULL || *password == '\0')
		    goto OK;
	    }
	    else
	    {
		if (password != NULL && strcmp(password, pp) == 0)
		    goto OK;
	    }
	    
	    break;
	}
    }

    fd_close(fd);
    return 0;

  OK:
    fd_close(fd);
    cip->auth.user = s_strdup(up);
    return 1;
}


int pm_auth(struct connectioninfo *cip,
	    const char *domain,
	    const char *type,
	    const char *user,
	    const char *password)
{
    hashentry_t *hep;
    int status;
    
    
    if (debug > 1)
	fprintf(stderr, "*** auth_file/pm_auth(%s,%s,%s,%s)\n",
		domain, type, user, password);


    hep = ht_lookup(auth_files_table, domain, strlen(domain));
    if (hep == NULL)
	return 0;

    status = check_file(hep->data, user, password, cip);

    ht_release(hep);
    
    return status;
}

