/*
** error.c
**
** Copyright (c) 1994-1995 Peter Eriksson <pen@signum.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "phttpd.h"


hashtable_t *error_page_table = NULL;

static void make_full_url(struct connectioninfo *cip, char *buf, int len)
{
    int i = 0;

    
    if (cip->hip->orig_url[0] == '/')
    {
	strcpy(buf, server_url);
	i = strlen(buf);
    }
    
    url_quote(cip->hip->orig_url, buf+i, len-i, "?", 0);
    i = strlen(buf);
    
    if (cip->hip->orig_request)
    {
	buf[i++] = '?';
	url_quote(cip->hip->orig_request, buf+i, len-i, "\"", 1);
	i = strlen(buf);
    }

    buf[i] = '\0';
}



int error_bad_request(struct connectioninfo *cip,
		      const char *buf)
{
    hashentry_t *hep;
    char buf1[2048];
    int len, code;
    
    
    hep = ht_lookup(error_page_table, "400", 3);
    if (hep == NULL)
	hep = ht_lookup(error_page_table, "*", 1);
	
    if (hep)
    {
	sprintf(buf1, "code=%d&url=", 400);
	len = strlen(buf1);

	make_full_url(cip, buf1+len, sizeof(buf1)-len);
	len = strlen(buf1);

	strcpy(buf1+len, "request=");
	len = strlen(buf1);
	
	url_quote(buf, buf1+len, sizeof(buf1)-len, "\"", 1);
	
	code = http_redirect(cip, hep->data, buf1, NULL, 302);
	ht_release(hep);
	return code;
    }
    else
	return http_error(cip, 400,
			  "Bad HTTP request:<BR><PRE>%s</PRE>\n", buf);
}



int error_not_found(struct connectioninfo *cip)
{
    char buf1[2048];
    hashentry_t *hep;
    int len, code;
    

    hep = ht_lookup(error_page_table, "404", 3);
    if (hep == NULL)
	hep = ht_lookup(error_page_table, "*", 1);

    if (hep)
    {
	sprintf(buf1, "code=%d&url=", 404);
	len = strlen(buf1);
	
	make_full_url(cip, buf1+len, sizeof(buf1)-len);

	code = http_redirect(cip, hep->data, buf1, NULL, 302);
	ht_release(hep);
	return code;
    }
    else
    {
	make_full_url(cip, buf1, sizeof(buf1));
	
	return http_error(cip, 404, 
		    "The requested URL, <A HREF=\"%s\">%s</A>, was not found.",
			  buf1, buf1);
    }
}


int error_access(struct connectioninfo *cip)
{
    char buf1[2048];
    hashentry_t *hep;
    int len, code;
    

    hep = ht_lookup(error_page_table, "403", 3);
    if (hep == NULL)
	hep = ht_lookup(error_page_table, "*", 1);

    if (hep)
    {
	sprintf(buf1, "code=%d&url=", 403);
	len = strlen(buf1);

	make_full_url(cip, buf1+len, sizeof(buf1)-len);

	code = http_redirect(cip, hep->data, buf1, NULL, 302);
	ht_release(hep);
	return code;
    }
    else
    {
	make_full_url(cip, buf1, sizeof(buf1));
    
	return http_error(cip, 403, 
			  "Acess Denied for URL <A HREF=\"%s\">%s</A>",
			  buf1);
    }
}

int error_system(struct connectioninfo *cip,
		 const char *prompt)
{
    return http_error(cip, 500, 
	       "A system error occured:<BR><PRE>%s: %s</PRE>",
	       prompt, strerror(errno));
}
