/*
** signals.c
**
** Copyright (c) 1994-1995 Peter Eriksson <pen@signum.se>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <signal.h>
#include <syslog.h>

#include "phttpd.h"

extern sema_t reconfig_sema;


static void sighup_handler()
{
    sema_post(&reconfig_sema);
}


static void sigusr1_handler()
{
    debug++;
}

static void sigusr2_handler()
{
    debug--;
}


static void sigterm_handler()
{
    syslog(LOG_NOTICE, "server terminating by SIGTERM request");
    exit(0);
}


void signals_setup(void)
{
    struct sigaction actb;

    
    actb.sa_handler = SIG_IGN;
    actb.sa_flags = 0;
    sigemptyset(&actb.sa_mask);
    sigaction(SIGPIPE, &actb, NULL);

    actb.sa_handler = SIG_IGN;
    actb.sa_flags = 0;
    sigemptyset(&actb.sa_mask);
    sigaction(SIGCHLD, &actb, NULL);

    actb.sa_handler = sighup_handler;
    actb.sa_flags = 0;
    sigemptyset(&actb.sa_mask);
    sigaction(SIGHUP, &actb, NULL);

    actb.sa_handler = sigusr1_handler;
    actb.sa_flags = 0;
    sigemptyset(&actb.sa_mask);
    sigaction(SIGUSR1, &actb, NULL);

    actb.sa_handler = sigusr2_handler;
    actb.sa_flags = 0;
    sigemptyset(&actb.sa_mask);
    sigaction(SIGUSR2, &actb, NULL);

    actb.sa_handler = sigterm_handler;
    actb.sa_flags = 0;
    sigemptyset(&actb.sa_mask);
    sigaction(SIGTERM, &actb, NULL);
}
